@@ -38,8 +38,8 @@ class FlatImages(datamodel.DataContainer):
       although they can be None (but shouldn't be)
 
     """
-    minimum_version = '1.1.0'
-    version = '1.1.0'
+    minimum_version = '1.1.1'
+    version = '1.1.1'
 
     # I/O
     output_to_disk = None  # This writes all items that are not None
@@ -60,6 +60,8 @@ class FlatImages(datamodel.DataContainer):
                                        descr='Mirrors SlitTraceSet mask for Flat-specific flags'),
                  'pixelflat_spec_illum': dict(otype=np.ndarray, atype=np.floating,
                                               descr='Relative spectral illumination'),
+                 'pixelflat_waveimg': dict(otype=np.ndarray, atype=np.floating,
+                                           descr='Waveimage for pixel flat'),
                  'illumflat_raw': dict(otype=np.ndarray, atype=np.floating,
                                        descr='Processed, combined illum flats'),
                  'illumflat_spat_bsplines': dict(otype=np.ndarray, atype=bspline.bspline,
@@ -71,8 +73,8 @@ class FlatImages(datamodel.DataContainer):
 
     def __init__(self, pixelflat_raw=None, pixelflat_norm=None, pixelflat_bpm=None,
                  pixelflat_model=None, pixelflat_spat_bsplines=None, pixelflat_spec_illum=None,
-                 illumflat_raw=None, illumflat_spat_bsplines=None, illumflat_bpm=None,
-                 PYP_SPEC=None, spat_id=None):
+                 pixelflat_waveimg=None, illumflat_raw=None, illumflat_spat_bsplines=None,
+                 illumflat_bpm=None, PYP_SPEC=None, spat_id=None):
         # Parse
         args, _, _, values = inspect.getargvalues(inspect.currentframe())
         d = dict([(k,values[k]) for k in args[1:]])
@@ -235,15 +237,6 @@ class FlatImages(datamodel.DataContainer):
                 msgs.warn("pixelflat has no spatial bspline fit - using the illumflat")
                 return self.illumflat_spat_bsplines
 
-    def get_pixelflat(self):
-        return self.pixelflat_norm
-
-    def get_spec_illum(self):
-        return self.pixelflat_spec_illum
-
-    def get_flat_model(self):
-        return self.pixelflat_model
-
     def fit2illumflat(self, slits, frametype='illum', initial=False, flexure_shift=None):
         """
 
@@ -331,11 +324,11 @@ class FlatImages(datamodel.DataContainer):
                              [(0.9, 1.1), (0.9, 1.1), None, None,
                               (0.8, 1.2), (0.9, 1.1), None])
         # Display frames
-        show_flats(image_list, wcs_match=wcs_match, slits=slits)
+        show_flats(image_list, wcs_match=wcs_match, slits=slits, waveimg=self.pixelflat_waveimg)
 
 
 
-class FlatField(object):
+class FlatField:
     """
     Builds pixel-level flat-field and the illumination flat-field.
 
@@ -382,7 +375,8 @@ class FlatField(object):
     master_type = 'Flat'
 
 
-    def __init__(self, rawflatimg, spectrograph, flatpar, slits, wavetilts, wv_calib, spat_illum_only=False):
+    def __init__(self, rawflatimg, spectrograph, flatpar, slits, wavetilts, wv_calib,
+                 spat_illum_only=False):
 
         # Defaults
         self.spectrograph = spectrograph
@@ -405,6 +399,7 @@ class FlatField(object):
         self.list_of_spat_bsplines = None
         self.spat_illum_only = spat_illum_only
         self.spec_illum = None      # Relative spectral illumination image
+        self.waveimg = None
 
         # Completed steps
         self.steps = []
@@ -451,15 +446,11 @@ class FlatField(object):
         Returns:
             :class:`FlatImages`:
         """
-        # Build the pixel flat (as needed)
-        #self.build_pixflat()
-
         # Fit it
         # NOTE: Tilts do not change and self.slits is updated internally.
         self.fit(spat_illum_only=self.spat_illum_only, debug=debug)
 
         if show:
-            # Global skysub is the first step in a new extraction so clear the channels here
             self.show(wcs_match=True)
 
         # Build the mask
@@ -479,6 +470,7 @@ class FlatField(object):
                               pixelflat_model=self.flat_model,
                               pixelflat_spat_bsplines=np.asarray(self.list_of_spat_bsplines),
                               pixelflat_bpm=bpmflats, pixelflat_spec_illum=self.spec_illum,
+                              pixelflat_waveimg=self.waveimg,
                               PYP_SPEC=self.spectrograph.name, spat_id=self.slits.spat_id)
 
     def build_mask(self):
@@ -507,7 +499,7 @@ class FlatField(object):
         image_list = zip([self.mspixelflat, self.msillumflat, self.rawflatimg.image, self.flat_model],
                          ['pixelflat', 'spat_illum', 'raw', 'model', 'spec_illum'],
                          [(0.9, 1.1), (0.9, 1.1), None, None, (0.8, 1.2)])
-        show_flats(image_list, wcs_match=wcs_match, slits=self.slits)
+        show_flats(image_list, wcs_match=wcs_match, slits=self.slits, waveimg=self.waveimg)
 
     def fit(self, spat_illum_only=False, debug=False):
         """
@@ -629,6 +621,16 @@ class FlatField(object):
         npoly = self.flatpar['twod_fit_npoly']
         saturated_slits = self.flatpar['saturated_slits']
 
+        # Build wavelength image -- not always used, but for convenience done here
+        slitmask = self.slits.slit_img(initial=True, 
+                                       flexure=self.wavetilts.spat_flexure)
+        tilts = self.wavetilts.fit2tiltimg(slitmask, 
+                                           flexure=self.wavetilts.spat_flexure)
+        waveimg = self.wv_calib.build_waveimg(
+            tilts, self.slits, spat_flexure=self.wavetilts.spat_flexure)
+        # Save to class attribute for inclusion in MasterFlat
+        self.waveimg = waveimg
+
         # Setup images
         nspec, nspat = self.rawflatimg.image.shape
         rawflat = self.rawflatimg.image
@@ -642,8 +644,15 @@ class FlatField(object):
         # set errors to just be 0.5 in the log
         ivar_log = gpm_log.astype(float)/0.5**2
 
+        # Get the non-linear count level
+        # TODO: This is currently hacked to deal with Mosaics
+        try:
+            nonlinear_counts = self.rawflatimg.detector.nonlinear_counts()
+        except:
+            nonlinear_counts = 1e10
         # Other setup
-        nonlinear_counts = self.spectrograph.nonlinear_counts(self.rawflatimg.detector)
+#        nonlinear_counts = self.spectrograph.nonlinear_counts(self.rawflatimg.detector)
+#        nonlinear_counts = self.rawflatimg.detector.nonlinear_counts()
 
         # TODO -- JFH -- CONFIRM THIS SHOULD BE ON INIT
         # It does need to be *all* of the slits
@@ -683,8 +692,17 @@ class FlatField(object):
         # Model each slit independently
         for slit_idx, slit_spat in enumerate(self.slits.spat_id):
             # Is this a good slit??
-            if self.slits.mask[slit_idx] != 0:
+            if self.slits.bitmask.flagged(self.slits.mask[slit_idx], flag=['SHORTSLIT', 'USERIGNORE', 'BADTILTCALIB']):
                 msgs.info('Skipping bad slit: {}'.format(slit_spat))
+                self.slits.mask[slit_idx] = self.slits.bitmask.turn_on(self.slits.mask[slit_idx], 'BADFLATCALIB')
+                continue
+            elif self.slits.bitmask.flagged(self.slits.mask[slit_idx], flag=['BOXSLIT']):
+                msgs.info('Skipping alignment slit: {}'.format(slit_spat))
+                continue
+            elif self.slits.bitmask.flagged(self.slits.mask[slit_idx], flag=['BADWVCALIB']) and \
+                    (self.flatpar['pixelflat_min_wave'] is not None or self.flatpar['pixelflat_max_wave'] is not None):
+                msgs.info('Skipping slit with bad wavecalib: {}'.format(slit_spat))
+                self.slits.mask[slit_idx] = self.slits.bitmask.turn_on(self.slits.mask[slit_idx], 'BADFLATCALIB')
                 continue
 
             msgs.info('Modeling the flat-field response for slit spat_id={}: {}/{}'.format(
@@ -798,15 +816,12 @@ class FlatField(object):
             #  the edges of the chip in spec direction
             # TODO: Can we add defaults to bspline_profile so that we
             #  don't have to instantiate invvar and profile_basis
-            try:
-                spec_bspl, spec_gpm_fit, spec_flat_fit, _, exit_status \
+            spec_bspl, spec_gpm_fit, spec_flat_fit, _, exit_status \
                     = fitting.bspline_profile(spec_coo_data, spec_flat_data, spec_ivar_data,
                                             np.ones_like(spec_coo_data), ingpm=spec_gpm_data,
                                             nord=4, upper=logrej, lower=logrej,
                                             kwargs_bspline={'bkspace': spec_samp_fine},
                                             kwargs_reject={'groupbadpix': True, 'maxrej': 5})
-            except:
-                embed(header='808 of flatfield')
 
             if exit_status > 1:
                 # TODO -- MAKE A FUNCTION
@@ -1050,6 +1065,7 @@ class FlatField(object):
             if exit_status > 1:
                 msgs.warn('Two-dimensional fit to flat-field data failed!  No higher order '
                           'flat-field corrections included in model of slit {0}!'.format(slit_spat))
+                self.slits.mask[slit_idx] = self.slits.bitmask.turn_on(self.slits.mask[slit_idx], 'BADFLATCALIB')
             else:
                 twod_model[twod_gpm] = twod_flat_fit[np.argsort(twod_srt)]
                 twod_gpm_out[twod_gpm] = twod_gpm_fit[np.argsort(twod_srt)]
@@ -1062,14 +1078,21 @@ class FlatField(object):
                                         * np.fmax(spec_model[onslit_tweak], 1.0)
 
             # Construct the pixel flat
-            #self.mspixelflat[onslit] = rawflat[onslit]/self.flat_model[onslit]
-            #self.mspixelflat[onslit_tweak] = 1.
             #trimmed_slitid_img_anew = self.slits.slit_img(pad=-trim, slitidx=slit_idx)
             #onslit_trimmed_anew = trimmed_slitid_img_anew == slit_spat
             self.mspixelflat[onslit_tweak] = rawflat[onslit_tweak]/self.flat_model[onslit_tweak]
             # TODO: Add some code here to treat the edges and places where fits
             #  go bad?
 
+            # Minimum wavelength?
+            if self.flatpar['pixelflat_min_wave'] is not None:
+                bad_wv = waveimg[onslit_tweak] < self.flatpar['pixelflat_min_wave'] 
+                self.mspixelflat[np.where(onslit_tweak)[0][bad_wv]] = 1.
+            # Maximum wavelength?
+            if self.flatpar['pixelflat_max_wave'] is not None:
+                bad_wv = waveimg[onslit_tweak] > self.flatpar['pixelflat_max_wave'] 
+                self.mspixelflat[np.where(onslit_tweak)[0][bad_wv]] = 1.
+
         # No need to continue if we're just doing the spatial illumination
         if spat_illum_only:
             return
@@ -1085,6 +1108,8 @@ class FlatField(object):
         # 100% to avoid creating edge effects, etc.
         self.mspixelflat = np.clip(self.mspixelflat, 0.5, 2.0)
 
+        
+
         # Calculate the relative spectral illumination, if requested
         if self.flatpar['slit_illum_relative']:
             self.spec_illum = self.spectral_illumination(twod_gpm_out, debug=debug)
@@ -1192,14 +1217,13 @@ class FlatField(object):
             mnmx_wv[slit_idx, 1] = np.max(waveimg[onslit_init])
 
         # Obtain relative spectral illumination
-        relscl_model = illum_profile_spectral(rawflat, waveimg, self.slits, slit_illum_ref_idx=self.flatpar['slit_illum_ref_idx'],
-                                              model=None, gpmask=gpm, skymask=None, trim=trim, flexure=flex)
-        # Invert
-        scale_model = 1 / (relscl_model + (relscl_model == 0))
-        return scale_model
+        return illum_profile_spectral(rawflat, waveimg, self.slits,
+                                      slit_illum_ref_idx=self.flatpar['slit_illum_ref_idx'],
+                                      model=None, gpmask=gpm, skymask=None, trim=trim,
+                                      flexure=flex)
 
 
-def show_flats(image_list, wcs_match=True, slits=None):
+def show_flats(image_list, wcs_match=True, slits=None, waveimg=None):
     """
     Interface to ginga to show a set of flat images
 
@@ -1211,6 +1235,7 @@ def show_flats(image_list, wcs_match=True, slits=None):
         spec_illum (`numpy.ndarray`_ or None):
         wcs_match (bool, optional):
         slits (:class:`pypeit.slittrace.SlitTraceSet`, optional):
+        waveimg (`numpy.ndarray`_ or None):
 
     Returns:
 
@@ -1227,7 +1252,7 @@ def show_flats(image_list, wcs_match=True, slits=None):
         # TODO: Add an option that shows the relevant stuff in a
         # matplotlib window.
         viewer, ch = display.show_image(img, chname=name, cuts=cut, wcs_match=wcs_match,
-                                        clear=clear)
+                                        waveimg=waveimg, clear=clear)
         if slits is not None:
             display.show_slits(viewer, ch, left[:, gpm], right[:, gpm],
                                slit_ids=slits.spat_id[gpm])
@@ -1417,3 +1442,6 @@ def merge(init_cls, merge_cls):
         dd[key] = namespace['val']
     # Construct the merged class
     return FlatImages(**dd)
+
+
+
