@@ -29,12 +29,12 @@ from pypeit import spec2dobj
 from pypeit import coadd2d
 from pypeit import specobjs
 from pypeit import slittrace
-from pypeit import reduce
+from pypeit import extraction
+from pypeit import find_objects
 from pypeit import calibrations
 from pypeit.display import display
 from pypeit.images import buildimage
 from pypeit.spectrographs.util import load_spectrograph
-from pypeit.core.parse import get_dnum
 from pypeit.core.wavecal import wvutils
 from pypeit import sensfunc
 from pypeit.core import flux_calib
@@ -111,18 +111,21 @@ def run_pair(A_files, B_files, caliBrate, spectrograph, det, parset, show=False,
     # Background Image?
     sciImg = sciImg.sub(buildimage.buildimage_fromlist(spectrograph, det, parset['scienceframe'], list(B_files), bpm=caliBrate.msbpm, slits=caliBrate.slits, ignore_saturation=False),
             parset['scienceframe']['process'])
-    # Instantiate Reduce object
+    # Instantiate FindObjects object
     # Required for pypeline specific object
     # At instantiaton, the fullmask in self.sciImg is modified
-    redux = reduce.Reduce.get_instance(sciImg, spectrograph, parset, caliBrate, 'science', ir_redux=True, show=show,
-                                       det=det)
 
-    # skymodel, objmodel, ivarmodel, outmask, sobjs, scaleimg, waveimg, tilts = redux.run(
-    #     std_trace=std_trace, return_negative=True, show_peaks=show)
+    # DP: Should find_negative be True here?
+    objFind = find_objects.FindObjects.get_instance(sciImg, spectrograph, parset, caliBrate, 'science',
+                                                    bkg_redux=True, find_negative=True, show=show)
 
-    global_sky, sobjs_obj, skymask = redux.run_objfind(std_trace=std_trace, show_peaks=show)
-    skymodel, objmodel, ivarmodel, outmask, sobjs, scaleimg, waveimg, tilts = redux.run_extraction(
-        global_sky, sobjs_obj, skymask, return_negative=True)
+    global_sky, sobjs_obj = objFind.run(std_trace=std_trace, show_peaks=show)
+
+    # Instantiate Extract object
+    extract = extraction.Extract.get_instance(sciImg, sobjs_obj, spectrograph, parset, caliBrate,
+                                              'science', bkg_redux=True, return_negative=True, show=show)
+    skymodel, objmodel, ivarmodel, \
+    outmask, sobjs, scaleimg, waveimg, tilts = extract.run(global_sky, sobjs_obj)
 
     # TODO -- Do this upstream
     # Tack on detector
@@ -132,11 +135,10 @@ def run_pair(A_files, B_files, caliBrate, spectrograph, det, parset, show=False,
     # Construct table of spectral flexure
     spec_flex_table = Table()
     spec_flex_table['spat_id'] = caliBrate.slits.spat_id
-    spec_flex_table['sci_spec_flexure'] = redux.slitshift
+    spec_flex_table['sci_spec_flexure'] = extract.slitshift
 
     # Construct the Spec2DObj with the positive image
-    spec2DObj_A = spec2dobj.Spec2DObj(det=det,
-                                      sciimg=sciImg.image,
+    spec2DObj_A = spec2dobj.Spec2DObj(sciimg=sciImg.image,
                                       ivarraw=sciImg.ivar,
                                       skymodel=skymodel,
                                       objmodel=objmodel,
@@ -150,15 +152,15 @@ def run_pair(A_files, B_files, caliBrate, spectrograph, det, parset, show=False,
                                       vel_corr=None,
                                       vel_type=parset['calibrations']['wavelengths']['refframe'],
                                       tilts=tilts,
-                                      slits=copy.deepcopy(caliBrate.slits))
+                                      slits=copy.deepcopy(caliBrate.slits),
+                                      maskdef_designtab=None)
     spec2DObj_A.process_steps = sciImg.process_steps
     all_spec2d = spec2dobj.AllSpec2DObj()
-    all_spec2d['meta']['ir_redux'] = True
-    all_spec2d[det] = spec2DObj_A
+    all_spec2d['meta']['bkg_redux'] = True
+    all_spec2d[spec2DObj_A.detname] = spec2DObj_A
 
     # Construct the Spec2DObj with the negative image
-    spec2DObj_B = spec2dobj.Spec2DObj(det=det,
-                                      sciimg=-sciImg.image,
+    spec2DObj_B = spec2dobj.Spec2DObj(sciimg=-sciImg.image,
                                       ivarraw=sciImg.ivar,
                                       skymodel=-skymodel,
                                       objmodel=-objmodel,
@@ -172,7 +174,8 @@ def run_pair(A_files, B_files, caliBrate, spectrograph, det, parset, show=False,
                                       vel_corr=None,
                                       vel_type=parset['calibrations']['wavelengths']['refframe'],
                                       tilts=tilts,
-                                      slits=copy.deepcopy(caliBrate.slits))
+                                      slits=copy.deepcopy(caliBrate.slits),
+                                      maskdef_designtab=None)
     return spec2DObj_A, spec2DObj_B
 
 
@@ -271,8 +274,13 @@ class QLKeckMOSFIRE(scriptbase.ScriptBase):
             msgs.error('Master frames not found.  Check that environment variable QL_MASTERS '
                        'points at the Master Calibs')
 
+        # Get detector (there's only one)
+        det = 1 # MOSFIRE has a single detector
+        detector = spectrograph.get_detector_par(det)
+        detname = detector.name
+
         # We need the platescale
-        platescale = spectrograph.get_detector_par(None, 1)['platescale']
+        platescale = detector['platescale']
         # Parse the offset information out of the headers. TODO in the future
         # get this out of fitstable
         dither_pattern, dither_id, offset_arcsec = spectrograph.parse_dither_pattern(files)
@@ -299,11 +307,10 @@ class QLKeckMOSFIRE(scriptbase.ScriptBase):
 
         ## Read in the master frames that we need
         ##
-        det = 1 # MOSFIRE has a single detector
         if std_spec1d_file is not None:
             # Get the standard trace if need be
             sobjs = specobjs.SpecObjs.from_fitsfile(std_spec1d_file)
-            this_det = sobjs.DET == det
+            this_det = sobjs.DET == detname
             if np.any(this_det):
                 sobjs_det = sobjs[this_det]
                 sobjs_std = sobjs_det.get_std()
@@ -314,7 +321,6 @@ class QLKeckMOSFIRE(scriptbase.ScriptBase):
             std_trace = None
 
         # Read in the msbpm
-        sdet = get_dnum(det, prefix=False)
         msbpm = spectrograph.bpm(A_files[0], det)
         # Read in the slits
         slits = slittrace.SlitTraceSet.from_file(slit_masterframe_name)
@@ -335,6 +341,7 @@ class QLKeckMOSFIRE(scriptbase.ScriptBase):
         caliBrate.msbpm = msbpm
         caliBrate.wavetilts = tilts_obj
         caliBrate.wv_calib = wv_calib
+        caliBrate.binning = f'{slits.binspec},{slits.binspat}'
 
         # Find the unique throw absolute value, which defines each MASK_NOD seqeunce
         #uniq_offsets, _ = np.unique(offset_arcsec, return_inverse=True)
@@ -398,7 +405,7 @@ class QLKeckMOSFIRE(scriptbase.ScriptBase):
                                              offsets=offsets_pixels, weights='uniform',
                                              spec_samp_fact=args.spec_samp_fact,
                                              spat_samp_fact=args.spat_samp_fact,
-                                             ir_redux=True, debug=args.show)
+                                             bkg_redux=True, debug=args.show)
         # Coadd the slits
         # TODO implement only_slits later
         coadd_dict_list = coadd.coadd(only_slits=None, interp_dspat=False)
@@ -442,8 +449,8 @@ class QLKeckMOSFIRE(scriptbase.ScriptBase):
             # reason
             mean, med, sigma = sigma_clipped_stats(imgminsky[imgminsky_gpm], sigma_lower=3.0,
                                                    sigma_upper=3.0)
-            chname_skysub = 'fluxed-skysub-det{:s}'.format(sdet) \
-                                if args.flux else 'skysub-det{:s}'.format(sdet)
+            chname_skysub = f'fluxed-skysub-{detname.lower()}' \
+                                if args.flux else f'skysub-{detname.lower()}'
             cuts_skysub = (med - 3.0 * sigma, med + 3.0 * sigma)
             cuts_resid = (-5.0, 5.0)
             #fits.writeto('/Users/joe/ginga_test.fits',imgminsky, overwrite=True)
@@ -461,7 +468,7 @@ class QLKeckMOSFIRE(scriptbase.ScriptBase):
             display.show_slits(viewer, ch_skysub, slit_left, slit_righ, slit_ids=slit_id)
 
             # SKRESIDS
-            chname_skyresids = 'sky_resid-det{:s}'.format(sdet)
+            chname_skyresids = f'sky_resid-{detname.lower()}'
             # sky residual map
             image = pseudo_dict['imgminsky']*np.sqrt(pseudo_dict['sciivar']) * pseudo_dict['inmask']
             viewer, ch_skyresids = display.show_image(image, chname_skyresids,
