@@ -15,29 +15,28 @@ from pypeit import specobjs
 from pypeit.spec2dobj import AllSpec2DObj
 from pypeit.core.collate import collate_spectra_by_source, SourceObject
 from pypeit.archive import ArchiveDir
-from pypeit.scripts.collate_1d import find_spec2d_from_spec1d,find_slits_to_exclude, exclude_source_objects, extract_id, get_metadata_reduced, get_metadata_coadded, get_report_metadata, find_archvie_files_from_spec1d
+from pypeit.scripts.collate_1d import find_spec2d_from_spec1d,find_slits_to_exclude, exclude_source_objects, get_report_metadata, flux, coadd
 from pypeit.spectrographs.util import load_spectrograph
 from pypeit.par import pypeitpar
 from pypeit.pypmsgs import PypeItError
+from pypeit.images.detector_container import DetectorContainer
+from pypeit import fluxcalibrate
+from pypeit import coadd1d
 
 from pypeit.tests.tstutils import data_path, cooked_required
 
-class mock_coadd:
-    pass
-
-def mock_get_instance():
-    pass
 
 
 class MockSpecObj:
-    def __init__(self, MASKDEF_OBJNAME, MASKDEF_ID, DET, RA, DEC, SPAT_PIXPOS, NAME, OPT_COUNTS=None, BOX_COUNTS=None):
+    def __init__(self, MASKDEF_OBJNAME, MASKDEF_ID, DET, RA, DEC, SPAT_PIXPOS, NAME, OPT_FLAM=None, OPT_COUNTS=None, BOX_COUNTS=None):
         self.MASKDEF_OBJNAME = MASKDEF_OBJNAME
         self.MASKDEF_ID = MASKDEF_ID
-        self.DET = DET
+        self.DET = DetectorContainer.get_name(DET)
         self.RA = RA
         self.SPAT_PIXPOS = SPAT_PIXPOS
         self.DEC = DEC
         self.NAME = NAME
+        self.OPT_FLAM = OPT_FLAM
         self.OPT_COUNTS = OPT_COUNTS
         self.BOX_COUNTS = BOX_COUNTS
 
@@ -86,16 +85,16 @@ class MockSpecObjs:
         # object4 also has boxcar counts and no opt_counts
 
         if file == "spec1d_file1":
-            self.specobjs = [MockSpecObj(MASKDEF_OBJNAME='object1',  MASKDEF_ID='1001', DET=1, RA=201.1517, DEC=27.3246, SPAT_PIXPOS=1234.0, NAME='SPAT1234_SLIT1234_DET01', OPT_COUNTS=np.zeros(100), BOX_COUNTS=np.zeros(100)),
-                             MockSpecObj(MASKDEF_OBJNAME='SERENDIP', MASKDEF_ID='1001', DET=1, RA=201.1522, DEC=27.3250, SPAT_PIXPOS=1334.0, NAME='SPAT1334_SLIT1234_DET01', OPT_COUNTS=np.zeros(100)),
-                             MockSpecObj(MASKDEF_OBJNAME='object2',  MASKDEF_ID='3002', DET=2, RA=201.0051, DEC=27.2228, SPAT_PIXPOS=5334.0, NAME='SPAT5334_SLIT4934_DET02', OPT_COUNTS=np.zeros(100)),
-                             MockSpecObj(MASKDEF_OBJNAME='object3',  MASKDEF_ID='3003', DET=3, RA=201.2517, DEC=27.3333, SPAT_PIXPOS=3233.0, NAME='SPAT3233_SLIT3235_DET03', OPT_COUNTS=np.zeros(100)),
+            self.specobjs = [MockSpecObj(MASKDEF_OBJNAME='object1',  MASKDEF_ID='1001', DET=1, RA=201.1517, DEC=27.3246, SPAT_PIXPOS=1234.0, NAME='SPAT1234_SLIT1234_DET01', OPT_COUNTS=np.zeros(100), OPT_FLAM=np.zeros(100), BOX_COUNTS=np.zeros(100)),
+                             MockSpecObj(MASKDEF_OBJNAME='SERENDIP', MASKDEF_ID='1001', DET=1, RA=201.1522, DEC=27.3250, SPAT_PIXPOS=1334.0, NAME='SPAT1334_SLIT1234_DET01', OPT_COUNTS=np.zeros(100), OPT_FLAM=np.zeros(100)),
+                             MockSpecObj(MASKDEF_OBJNAME='object2',  MASKDEF_ID='3002', DET=2, RA=201.0051, DEC=27.2228, SPAT_PIXPOS=5334.0, NAME='SPAT5334_SLIT4934_DET02', OPT_COUNTS=np.zeros(100), OPT_FLAM=np.zeros(100)),
+                             MockSpecObj(MASKDEF_OBJNAME='object3',  MASKDEF_ID='3003', DET=3, RA=201.2517, DEC=27.3333, SPAT_PIXPOS=3233.0, NAME='SPAT3233_SLIT3235_DET03', OPT_COUNTS=np.zeros(100), OPT_FLAM=np.zeros(100)),
                              MockSpecObj(MASKDEF_OBJNAME='object3',  MASKDEF_ID='3003', DET=3, RA=201.2517, DEC=27.3333, SPAT_PIXPOS=3232.0, NAME='SPAT3232_SLIT3235_DET03'),
-                             MockSpecObj(MASKDEF_OBJNAME='object3',  MASKDEF_ID='3003', DET=5, RA=201.2517, DEC=27.3333, SPAT_PIXPOS=3236.0, NAME='SPAT3236_SLIT3245_DET05', OPT_COUNTS=np.zeros(100)),
-                             MockSpecObj(MASKDEF_OBJNAME='object1',  MASKDEF_ID='1001', DET=7, RA=201.1517, DEC=27.3246, SPAT_PIXPOS=1233.0, NAME='SPAT1233_SLIT1235_DET07', OPT_COUNTS=np.zeros(100), BOX_COUNTS=np.zeros(100)),
-                             MockSpecObj(MASKDEF_OBJNAME='SERENDIP', MASKDEF_ID='1001', DET=7, RA=201.1520, DEC=27.3249, SPAT_PIXPOS=1336.0, NAME='SPAT1336_SLIT1235_DET07', OPT_COUNTS=np.zeros(100))]
+                             MockSpecObj(MASKDEF_OBJNAME='object3',  MASKDEF_ID='3003', DET=5, RA=201.2517, DEC=27.3333, SPAT_PIXPOS=3236.0, NAME='SPAT3236_SLIT3245_DET05', OPT_COUNTS=np.zeros(100), OPT_FLAM=np.zeros(100)),
+                             MockSpecObj(MASKDEF_OBJNAME='object1',  MASKDEF_ID='1001', DET=7, RA=201.1517, DEC=27.3246, SPAT_PIXPOS=1233.0, NAME='SPAT1233_SLIT1235_DET07', OPT_COUNTS=np.zeros(100), OPT_FLAM=np.zeros(100), BOX_COUNTS=np.zeros(100)),
+                             MockSpecObj(MASKDEF_OBJNAME='SERENDIP', MASKDEF_ID='1001', DET=7, RA=201.1520, DEC=27.3249, SPAT_PIXPOS=1336.0, NAME='SPAT1336_SLIT1235_DET07', OPT_COUNTS=np.zeros(100), OPT_FLAM=np.zeros(100))]
         else:
-            self.specobjs = [MockSpecObj(MASKDEF_OBJNAME='object3',  MASKDEF_ID='3003', DET=3, RA=201.2517, DEC=27.3333, SPAT_PIXPOS=3234.0, NAME='SPAT3234_SLIT3236_DET03', OPT_COUNTS=np.zeros(100)),
+            self.specobjs = [MockSpecObj(MASKDEF_OBJNAME='object3',  MASKDEF_ID='3003', DET=3, RA=201.2517, DEC=27.3333, SPAT_PIXPOS=3234.0, NAME='SPAT3234_SLIT3236_DET03', OPT_FLAM=np.zeros(100), OPT_COUNTS=np.zeros(100)),
                              MockSpecObj(MASKDEF_OBJNAME='object4',  MASKDEF_ID='4004', DET=3, RA=201.0052, DEC=27.2418, SPAT_PIXPOS=6250.0, NAME='SPAT6250_SLIT6235_DET03', BOX_COUNTS=np.zeros(100)),
                              MockSpecObj(MASKDEF_OBJNAME='object4',  MASKDEF_ID='4004', DET=5, RA=201.0052, DEC=27.2418, SPAT_PIXPOS=6256.0, NAME='SPAT6256_SLIT6245_DET05', BOX_COUNTS=np.zeros(100)),
                              MockSpecObj(MASKDEF_OBJNAME='SERENDIP', MASKDEF_ID='4004', DET=5, RA=201.0056, DEC=27.2419, SPAT_PIXPOS=6934.0, NAME='SPAT6934_SLIT6245_DET05', BOX_COUNTS=np.zeros(100)),
@@ -377,95 +376,6 @@ def test_find_spec2d_from_spec1d(tmp_path):
     with pytest.raises(PypeItError):
         result = find_spec2d_from_spec1d(spec1d_names)
 
-def test_extract_id():
-
-    mock_header_with_koaid = {'KOAID': 'DE.20121017.20019.fits'}
-    mock_header_with_koaid_in_filename = {'FILENAME': 'DE.20121017.20019blah.fits'}
-    mock_header_with_short_non_koa_filename = {'FILENAME':'short.fits'}
-    mock_header_with_long_non_koa_filename  =  {'FILENAME': 'file_name_longer_than_17.fits'}
-
-    assert extract_id(mock_header_with_koaid) == 'DE.20121017.20019.fits'
-    assert extract_id(mock_header_with_koaid_in_filename) == 'DE.20121017.20019.fits'
-    assert extract_id(mock_header_with_short_non_koa_filename) == 'short.fits'
-    assert extract_id(mock_header_with_long_non_koa_filename) == 'file_name_longer_than_17.fits'
-
-def test_get_metadata_reduced(monkeypatch):
-
-    monkeypatch.setattr(fits, "getheader", mock_header)
-    mock_files = (os.path.join('Science', 'spec1d_file1'), os.path.join('Science', 'spec1d_file1.txt'),  os.path.join('Science', 'spec2d_file1.fits'),'test.pypeit')
-    dest_files = (os.path.join('2019B_TEST1', 'spec1d_file1'), os.path.join('2019B_TEST1', 'spec1d_file1.txt'), os.path.join('2019B_TEST1', 'spec2d_file1.fits'), os.path.join('2019B_TEST1', 'test.pypeit'))
-    (metadata_rows, files_to_copy) = get_metadata_reduced(['DISPNAME', 'TESTKEY'], mock_files)
-    header = mock_header('spec1d_file1')
-    assert len(metadata_rows) == 1
-    assert len(metadata_rows[0]) == 7
-    assert metadata_rows[0] == [header['FILENAME'] + ".fits", dest_files[0], dest_files[1], dest_files[2], dest_files[3], '830G', None]
-
-    # Convert the iterable files_to_copy to a list
-    files_to_copy = list(files_to_copy)
-    assert files_to_copy[0] == (mock_files[0],   dest_files[0])
-    assert files_to_copy[1] == (mock_files[1],   dest_files[1])
-    assert files_to_copy[2] == (mock_files[2],   dest_files[2])
-    assert files_to_copy[3] == (mock_files[3],   dest_files[3])
-
-    spectrograph = load_spectrograph('keck_deimos')
-    source_object = SourceObject(mock_specobjs("spec1d_file1").specobjs[0], header, "spec1d_file1", spectrograph, 'ra/dec')
-
-    assert (None, None) == get_metadata_reduced(['DISPNAME', 'TESTKEY'], source_object)
-
-def test_get_coadded_metadata(monkeypatch):
-
-    spectrograph = load_spectrograph('keck_deimos')
-    filenames = ['spec1d_file1', 'spec1d_file2']
-    specobjs_file1 = mock_specobjs(filenames[0])
-    specobjs_file2 = mock_specobjs(filenames[1])
-    header_file1 = mock_header(filenames[0])
-    header_file2 = mock_header(filenames[1])
-    file1_objects = [3,  # 'SPAT3233_SLIT3235_DET03'
-                     5,] # 'SPAT3236_SLIT3245_DET05'
-                     
-    file2_objects = [0,  # 'SPAT3234_SLIT3236_DET03'
-                     4,] # 'SPAT3237_SLIT3246_DET05'
-    
-    source_object = SourceObject(specobjs_file1.specobjs[file1_objects[0]], 
-                                 header_file1, 
-                                 filenames[0], 
-                                 spectrograph, 
-                                 'ra/dec')
-    source_object.coaddfile = "/user/test/coaddfile.fits"
-
-    # The coadded file's mock header doesn't have PROGID and SEMESTER, so it will be in the
-    # archive's base directory
-    dest_file = os.path.basename(source_object.coaddfile)
-    for object in file1_objects[1:]:
-        source_object.spec_obj_list.append(specobjs_file1.specobjs[object])
-        source_object.spec1d_file_list.append(filenames[0])
-        source_object.spec1d_header_list.append(header_file1)
-
-    for object in file2_objects:
-        source_object.spec_obj_list.append(specobjs_file2.specobjs[object])
-        source_object.spec1d_file_list.append(filenames[1])
-        source_object.spec1d_header_list.append(header_file2)
-
-    monkeypatch.setattr(fits, "getheader", mock_header)
-
-    (metadata_rows, files_to_copy) = get_metadata_coadded(['DISPNAME','MJD', 'GUIDFHWM'],
-                                                          ['MASKDEF_OBJNAME', 'NAME'],
-                                                          source_object)
-
-    assert len(metadata_rows) == 4
-    # The spec1d_file2's header doesn't have the semester and prog id, so it's path will be the root directory
-    # of the archive
-    assert metadata_rows[0] == [dest_file, 'object3', 'SPAT3233_SLIT3235_DET03', 'DE.20100913.22358.fits', os.path.join('2019B_TEST1', 'spec1d_file1'), '830G', '58878.0', None]
-    assert metadata_rows[1] == [dest_file, 'object3', 'SPAT3236_SLIT3245_DET05', 'DE.20100913.22358.fits', os.path.join('2019B_TEST1', 'spec1d_file1'), '830G', '58878.0', None]
-    assert metadata_rows[2] == [dest_file, 'object3', 'SPAT3234_SLIT3236_DET03', 'DE.20100914.12358.fits', 'spec1d_file2', '830G', '58879.0', None]
-    assert metadata_rows[3] == [dest_file, 'object3', 'SPAT3237_SLIT3246_DET05', 'DE.20100914.12358.fits', 'spec1d_file2', '830G', '58879.0', None]
-    assert files_to_copy[0][0] == source_object.coaddfile
-    assert files_to_copy[0][1] == dest_file
-    
-    assert (None, None) ==  get_metadata_coadded(['DISPNAME', 'MJD', 'GUIDFHWM'],
-                                                 ['MASKDEF_OBJNAME', 'NAME'],
-                                                 "afilename")
-
 def test_get_report_metadata(monkeypatch):
 
     spectrograph = load_spectrograph('keck_deimos')
@@ -514,51 +424,128 @@ def test_get_report_metadata(monkeypatch):
                                                 ['MASKDEF_OBJNAME', 'NAME'],
                                                 "afilename")
 
-def test_find_archive_files_from_spec1d(tmp_path):
-
-    # Write dummy files to avoid dependency on Cooked
-    spec1d_names = [str(tmp_path / 'Science' / 'spec1d_test-name.1.fits'), str(tmp_path / 'Science' / 'spec1d_test-name.2.fits')]
-    related_files = [str(tmp_path / 'test1.pypeit'), str(tmp_path / 'test2.pypeit'), str(tmp_path / 'pypeit' / 'test3.pypeit'), str(tmp_path / 'Science' / 'spec1d_test-name.1.txt')]
-
-    for file in spec1d_names + related_files:
-        os.makedirs(os.path.dirname(file), exist_ok=True)
-        with open(file, "w") as f:
-            print("Dummy fits file", file=f)
+def test_flux(monkeypatch):
     
-    mock_par = {'collate1d': {"pypeit_file": None}}
-    # This will raise an exception because of too many .pypeit files
-    # find one txt file and give a warning message about one missing file. It should
-    # also find the one pypeit file
-    with pytest.raises(PypeItError, match = "found more than one file."):
-        (text_files, pypeit_files, missing_msgs) = find_archvie_files_from_spec1d(mock_par, spec1d_names)
+    def mock_get_header(file):
+        if file == "fail_grating.fits":
+            return {"DISPNAME": "Unknown"}
+        else: 
+            return {"DISPNAME": "600ZD" }
 
-    # Remove the extra .pypeit file, This will work but will give a warning about the missing
-    # .txt file
-    os.unlink(related_files[1])
-    (text_files, pypeit_files, missing_msgs) = find_archvie_files_from_spec1d(mock_par, spec1d_names)
-    assert len(text_files) == 2
-    assert text_files[0] == related_files[3]
-    assert text_files[1] == None
-    assert len(pypeit_files) == 2
-    assert pypeit_files[0] == related_files[0]
-    assert pypeit_files[1] == related_files[0]
-    assert len(missing_msgs) == 1
-    assert missing_msgs[0] == f"Could not archive matching text file for {spec1d_names[1]}, file not found."
+    def mock_get_flux_calib_instance(spec1d_files, sens_files, par):
+        if spec1d_files[0] == "fail_flux.fits":
+            raise PypeItError("Test failure")
+        else:
+            # The collate_1d caller doesn't use the output, it just
+            # depends on the side effect of fluxing
+            return None 
 
-    # Remove the remaining the pypeit file, which should raise an exception
-    os.unlink(related_files[0])
-    with pytest.raises(PypeItError, match = f"Could not archive matching .pypeit file for {re.escape(spec1d_names[0])}, file not found."):
-        (text_files, pypeit_files, missing_msgs) = find_archvie_files_from_spec1d(mock_par, spec1d_names)
+    # Test success
+    with monkeypatch.context() as m:
+        monkeypatch.setattr(fits, "getheader", mock_get_header)
+        monkeypatch.setattr(fluxcalibrate.FluxCalibrate, "get_instance", mock_get_flux_calib_instance)
 
-    # Specify a custom location for the pypeit file
-    mock_par = {'collate1d': {"pypeit_file": related_files[2]}}
-    (text_files, pypeit_files, missing_msgs) = find_archvie_files_from_spec1d(mock_par, spec1d_names)
-    assert len(pypeit_files) == 2
-    assert pypeit_files[0] == related_files[2]
-    assert pypeit_files[1] == related_files[2]
+        spectrograph = load_spectrograph('keck_deimos')
+        # This could break if we start supporting it
+        unsupported_spectrograph = load_spectrograph('shane_kast_red')
 
-    # Remove the custom .pypeit file, which should raise an exception
-    os.unlink(related_files[2])
-    with pytest.raises(PypeItError, match = f"Could not archive passed in .pypeit file {re.escape(related_files[2])}, file not found."):
-        (text_files, pypeit_files, missing_msgs) = find_archvie_files_from_spec1d(mock_par, spec1d_names)
+        par = pypeitpar.PypeItPar()
+        par['fluxcalib'] = pypeitpar.FluxCalibratePar()
+
+        # Test success
+        failed_messages = []
+        flux(par, spectrograph, ["no_fail.fits"], failed_messages)
+
+        assert len(failed_messages) == 0
+
+        # Test failure due to no archived sensfunc
+        flux(par, spectrograph, ["fail_grating.fits"], failed_messages)
+        assert failed_messages[0] == "Could not find archived sensfunc to flux fail_grating.fits, skipping it."
+
+        failed_messages = []
+
+        # Test failure in fluxing
+        flux(par, spectrograph, ["fail_flux.fits"], failed_messages)
+        assert failed_messages[0] == "Failed to flux calibrate fail_flux.fits, skipping it."
+        
+        # Test failure due to unsupported spectrograph
+        with pytest.raises(PypeItError):
+            flux(par, unsupported_spectrograph, ["600ZD_file.fits"], failed_messages)            
+
+def test_coadd(monkeypatch):
+
+    class mock_coadd:
+
+        def run(self):
+            return
+
+        def save(self, file):
+            return
+
+    def mock_get_instance(*args, **kwargs):
+        return mock_coadd()
+
+    with monkeypatch.context() as m:
+        monkeypatch.setattr(coadd1d.CoAdd1D, "get_instance", mock_get_instance)
+        par = pypeitpar.PypeItPar()
+        par['collate1d'] = pypeitpar.Collate1DPar()
+        par['coadd1d'] = pypeitpar.Coadd1DPar()
+        spectrograph = load_spectrograph('keck_deimos')
+
+        filenames = ['spec1d_file1', 'spec1d_file2']
+        specobjs_file1 = mock_specobjs(filenames[0])
+        specobjs_file2 = mock_specobjs(filenames[1])
+        header_file1 = mock_header(filenames[0])
+        header_file2 = mock_header(filenames[1])
+
+        # Both source object 1's SpecObj objects will have OPT_FLAM
+        file1_objects = [3,  # 'SPAT3233_SLIT3235_DET03'
+                         5,] # 'SPAT3236_SLIT3245_DET05'
+                            
+        # Only one of source object 2's SpecObj objects will have OPT_FLAM
+        file2_objects = [0,  # 'SPAT3234_SLIT3236_DET03'
+                         4,] # 'SPAT3237_SLIT3246_DET05'
+
+        source_object1 = SourceObject(specobjs_file1.specobjs[file1_objects[0]], 
+                                      header_file1, 
+                                      filenames[0], 
+                                      spectrograph, 
+                                      'ra/dec')
+
+        for object in file1_objects[1:]:
+            source_object1.spec_obj_list.append(specobjs_file1.specobjs[object])
+            source_object1.spec1d_file_list.append(filenames[0])
+            source_object1.spec1d_header_list.append(header_file1)
+
+        source_object2 = SourceObject(specobjs_file2.specobjs[file2_objects[0]], 
+                                      header_file2, 
+                                      filenames[1], 
+                                      spectrograph, 
+                                      'ra/dec')
+
+        for object in file2_objects:
+            source_object2.spec_obj_list.append(specobjs_file2.specobjs[object])
+            source_object2.spec1d_file_list.append(filenames[1])
+            source_object2.spec1d_header_list.append(header_file2)
+
+        # Test with out using fluxed data
+        par['collate1d']['ignore_flux'] = True
+        par['coadd1d']['flux_value'] = True
+        coadd(par, source_object1)
+
+        assert par['coadd1d']['flux_value'] == False
+
+        # Test using fluxed data
+        par['collate1d']['ignore_flux'] = False
+        par['coadd1d']['flux_value'] = False
+        coadd(par, source_object1)
+
+        assert par['coadd1d']['flux_value'] == True
+
+        # Test not using fluxed data because not all SpecObj objects had flux data
+        par['collate1d']['ignore_flux'] = False
+        par['coadd1d']['flux_value'] = True
+        coadd(par, source_object2)
+
+        assert par['coadd1d']['flux_value'] == False
 
