@@ -2,7 +2,9 @@
 Module to run tests on scripts
 """
 import os
+from pkg_resources import resource_filename
 import shutil
+from pathlib import Path
 
 import numpy as np
 import pytest
@@ -11,9 +13,9 @@ import matplotlib
 from IPython import embed
 matplotlib.use('agg')  # For Travis
 
-#import warnings
-#warnings.simplefilter('error', FutureWarning)
+from astropy.io import fits
 
+from pypeit.scripts import parse_slits
 from pypeit import scripts
 from pypeit.tests.tstutils import dev_suite_required, cooked_required, data_path
 from pypeit.display import display
@@ -22,10 +24,13 @@ from pypeit import utils
 from pypeit import io
 from pypeit import wavecalib
 from pypeit import coadd1d
+from pypeit import fluxcalibrate
+from pypeit import onespec
 
 from pypeit.pypeitsetup import PypeItSetup
 from pypeit.pypmsgs import PypeItError
 
+
 @dev_suite_required
 def test_quicklook():
     # The following needs the LRISb calibration files to be
@@ -87,7 +92,7 @@ def test_trace_edges():
             scripts.trace_edges.TraceEdges.parse_args(['-f', pypeit_file]))
 
     # Define the edges master file (HARDCODED!!)
-    trace_file = os.path.join(outdir, 'Masters', 'MasterEdges_A_1_01.fits.gz')
+    trace_file = os.path.join(outdir, 'Masters', 'MasterEdges_A_1_DET01.fits.gz')
 
     # Check that the correct number of traces were found
     edges = edgetrace.EdgeTraceSet.from_file(trace_file)
@@ -130,7 +135,7 @@ def test_trace_add_rm():
             scripts.trace_edges.TraceEdges.parse_args(['-f', pypeit_file]))
 
     # Define the edges master file (HARDCODED!!)
-    trace_file = os.path.join(outdir, 'Masters', 'MasterEdges_A_1_01.fits.gz')
+    trace_file = os.path.join(outdir, 'Masters', 'MasterEdges_A_1_DET01.fits.gz')
 
     # Check that the correct number of traces were found
     edges = edgetrace.EdgeTraceSet.from_file(trace_file)
@@ -182,20 +187,54 @@ def test_chk_edges():
 
 
 @cooked_required
-def test_view_fits():
-    """ Only test the list option
+def test_view_fits_list():
+    """ Test the list option
     """
     spec_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science',
                             'spec1d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
     #spec_file = data_path('spec1d_b27-J1217p3905_KASTb_2015May20T045733.560.fits')
-    pargs = scripts.view_fits.ViewFits.parse_args([spec_file, '--list', 'shane_kast_blue'])
-    # TODO: Should this test be calling the main function?
+    pargs = scripts.view_fits.ViewFits.parse_args(['shane_kast_blue', spec_file, '--list'])
+    scripts.view_fits.ViewFits.main(pargs)
+
+
+@cooked_required
+def test_view_fits_proc_fail():
+    """ Test that it fails when trying to proc an output pypeit image
+    """
+    spec_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science',
+                            'spec2d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
+    #spec_file = data_path('spec1d_b27-J1217p3905_KASTb_2015May20T045733.560.fits')
+    pargs = scripts.view_fits.ViewFits.parse_args(['shane_kast_blue', spec_file, '--proc'])
+    with pytest.raises(PypeItError):
+        scripts.view_fits.ViewFits.main(pargs)
+
+
+@dev_suite_required
+def test_view_fits_proc():
+    """ Test that it works on a raw image
+    """
+    spec_file = Path(os.getenv('PYPEIT_DEV')).resolve() / 'RAW_DATA' / 'shane_kast_blue' \
+                    / '830_3460_d46' / 'b100.fits.gz'
+    pargs = scripts.view_fits.ViewFits.parse_args(['shane_kast_blue', str(spec_file), '--proc'])
+    scripts.view_fits.ViewFits.main(pargs)
+
+
+@dev_suite_required
+def test_view_fits_mosaic():
+    """ Test the list option
+    """
+    file = os.path.join(os.environ['PYPEIT_DEV'], 'RAW_DATA', 'gemini_gmos', 'GN_HAM_R400_885',
+                        'N20190205S0035.fits')
+    pargs = scripts.view_fits.ViewFits.parse_args(['gemini_gmos_north_ham', file,
+                                                   '--det', 'mosaic',
+                                                   '--proc'])
+    scripts.view_fits.ViewFits.main(pargs)
 
 
 @cooked_required
 def test_chk_flat():
     mstrace_root = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'shane_kast_blue',
-                                'MasterFlat_A_1_01.fits')
+                                'MasterFlat_A_1_DET01.fits')
     # Ginga needs to be open in RC mode
     display.connect_to_ginga(raise_err=True, allow_new=True)
     #
@@ -206,12 +245,13 @@ def test_chk_flat():
 @cooked_required
 def test_chk_wavecalib():
     ms_root = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'shane_kast_blue',
-                                'MasterWaveCalib_A_1_01.fits')
+                           'MasterWaveCalib_A_1_DET01.fits')
     #
     pargs = scripts.chk_wavecalib.ChkWaveCalib.parse_args([ms_root])
     scripts.chk_wavecalib.ChkWaveCalib.main(pargs)
 
 
+# NOTE: May fail if DetectorContainer datamodel changes.
 def test_coadd1d_1():
     """
     Test basic coadd using shane_kast_blue
@@ -228,10 +268,12 @@ def test_coadd1d_1():
     scripts.coadd_1dspec.CoAdd1DSpec.main(
             scripts.coadd_1dspec.CoAdd1DSpec.parse_args([coadd_ifile, '--test_spec_path',
                                                          data_path('')]))
-
     hdu = io.fits_open(coadd_ofile)
     assert hdu[1].header['EXT_MODE'] == 'OPT'
     assert hdu[1].header['FLUXED'] is False
+    # Test that the output file is kosher and contains the right quantities
+    spec = onespec.OneSpec.from_file(coadd_ofile)
+    assert spec.wave.shape == spec.wave_grid_mid.shape
 
     # Clean up
     hdu.close()
@@ -239,6 +281,7 @@ def test_coadd1d_1():
     os.remove(coadd_ofile)
 
 
+# NOTE: May fail if DetectorContainer datamodel changes.
 def test_coadd1d_2():
     """
     Test combining Echelle
@@ -260,18 +303,24 @@ def test_coadd1d_2():
     assert hdu[1].header['EXT_MODE'] == 'OPT'
     assert hdu[1].header['FLUXED'] is False
 
+    # Test that the output file is kosher and contains the right quantities
+    spec = onespec.OneSpec.from_file(coadd_ofile)
+    assert spec.wave.shape == spec.wave_grid_mid.shape
+
     # Clean up
     hdu.close()
     os.remove(parfile)
     os.remove(coadd_ofile)
 
+#test_coadd1d_2()
+
 
 @cooked_required
 def test_identify():
     arc_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'shane_kast_blue',
-                             'MasterArc_A_1_01.fits')
+                             'MasterArc_A_1_DET01.fits')
     slits_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'shane_kast_blue',
-                            'MasterSlits_A_1_01.fits.gz')
+                            'MasterSlits_A_1_DET01.fits.gz')
     # Just list
     pargs = scripts.identify.Identify.parse_args([arc_file, slits_file, '--test'])
     arcfitter = scripts.identify.Identify.main(pargs)
@@ -295,12 +344,12 @@ def test_identify():
 
     waveCalib = wavecalib.WaveCalib(nslits=1, wv_fits=np.atleast_1d(arcfitter._fitdict['WaveFit']),
                               arc_spectra=np.atleast_2d(arcfitter.specdata).T,
-                              spat_ids=np.atleast_1d(arcfitter._slit),
+                              spat_ids=np.atleast_1d(int(arcfitter._spatid)),
                               PYP_SPEC='shane_kast_blue',
                               )
 
     # If you touch the following line, you probably need to update the call in scripts/identify.py
-    arcfitter.store_solution(final_fit, '', 1, force_save=True, wvcalib=waveCalib)
+    arcfitter.store_solution(final_fit, 1, rmstol=0.1, force_save=True, wvcalib=waveCalib)
 
     # Test we can read it
     tmp = wavecalib.WaveCalib.from_file('wvcalib.fits')
@@ -310,6 +359,7 @@ def test_identify():
     os.remove('wvarxiv.fits')
     os.remove('wvcalib.fits')
 
+
 @dev_suite_required
 def test_obslog():
     # Define the output directories (HARDCODED!!)
@@ -327,11 +377,29 @@ def test_obslog():
     # Clean up
     shutil.rmtree(setupdir)
 
+
+@cooked_required
+def test_compare_sky():
+    spec_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science',
+                             'spec1d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
+    sky_file = os.path.join(resource_filename('pypeit', 'data/sky_spec/'),
+                                              'sky_kastb_600.fits')
+
+    # Running in `test` mode for boxcar extraction
+    pargs = scripts.compare_sky.CompareSky.parse_args([spec_file, sky_file, '--test'])
+    scripts.compare_sky.CompareSky.main(pargs)
+
+    # Running in `test` mode for optimal extraction
+    pargs = scripts.compare_sky.CompareSky.parse_args([spec_file, sky_file, '--test',
+                                                       '--optimal'])
+    scripts.compare_sky.CompareSky.main(pargs)
+
+
 @cooked_required
 def test_collate_1d(tmp_path, monkeypatch):
 
     # Build up arguments for testing command line parsing
-    args = ['--dry_run', '--archive_dir', '/archive', '--outdir', '/outdir2', '--pypeit_file', 'file.pypeit', '--match', 'ra/dec', '--exclude_slit_bm', 'BOXSLIT', '--exclude_serendip']
+    args = ['--dry_run', '--ignore_flux', '--flux', '--outdir', '/outdir2', '--match', 'ra/dec', '--exclude_slit_bm', 'BOXSLIT', '--exclude_serendip']
     spec1d_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science', 'spec1d_b27*')
     spec1d_args = ['--spec1d_files', spec1d_file]
     tol_args = ['--tolerance', '0.03d']
@@ -348,9 +416,9 @@ def test_collate_1d(tmp_path, monkeypatch):
         print("ex_value = BOX", file=f)
         print("[collate1d]", file=f)
         print("dry_run = False", file=f)
+        print("flux = False", file=f)
         print("outdir = /outdir", file=f)
-        print("pypeit_file = otherfile.pypeit", file=f)
-        print("archive_root = /foo/bar", file=f)
+        print("ignore_flux = False", file=f)
         print("tolerance = 4.0", file=f)
         print("match_using = 'pixel'", file=f)
         print("exclude_slit_trace_bm = BADREDUCE", file=f)
@@ -381,9 +449,7 @@ def test_collate_1d(tmp_path, monkeypatch):
     parsed_args = scripts.collate_1d.Collate1D.parse_args(args + tol_args + spec1d_args)
     params, spectrograph, expanded_spec1d_files = scripts.collate_1d.build_parameters(parsed_args)
     assert params['collate1d']['dry_run'] is True
-    assert params['collate1d']['archive_root'] == '/archive'
     assert params['collate1d']['outdir'] == '/outdir2'
-    assert params['collate1d']['pypeit_file'] == 'file.pypeit'
     assert params['collate1d']['match_using'] == 'ra/dec'
     assert params['collate1d']['tolerance'] == '0.03d'
     assert params['collate1d']['exclude_slit_trace_bm'] == ['BOXSLIT']
@@ -396,9 +462,9 @@ def test_collate_1d(tmp_path, monkeypatch):
     parsed_args = scripts.collate_1d.Collate1D.parse_args([config_file_full])
     params, spectrograph, expanded_spec1d_files = scripts.collate_1d.build_parameters(parsed_args)
     assert params['collate1d']['dry_run'] is False
-    assert params['collate1d']['archive_root'] == '/foo/bar'
     assert params['collate1d']['outdir'] == '/outdir'
-    assert params['collate1d']['pypeit_file'] == 'otherfile.pypeit'
+    assert params['collate1d']['ignore_flux'] == False
+    assert params['collate1d']['flux'] == False
     assert params['collate1d']['tolerance'] == 4.0
     assert params['collate1d']['match_using'] == 'pixel'
     assert params['collate1d']['exclude_slit_trace_bm'] == 'BADREDUCE'
@@ -412,9 +478,9 @@ def test_collate_1d(tmp_path, monkeypatch):
                                                           + [config_file_full])
     params, spectrograph, expanded_spec1d_files = scripts.collate_1d.build_parameters(parsed_args)
     assert params['collate1d']['dry_run'] is True
-    assert params['collate1d']['archive_root'] == '/archive'
     assert params['collate1d']['outdir'] == '/outdir2'
-    assert params['collate1d']['pypeit_file'] == 'file.pypeit'
+    assert params['collate1d']['ignore_flux'] == True
+    assert params['collate1d']['flux'] == True
     assert params['collate1d']['tolerance'] == '0.03d'
     assert params['collate1d']['match_using'] == 'ra/dec'
     assert params['collate1d']['exclude_slit_trace_bm'] == ['BOXSLIT']
@@ -449,60 +515,25 @@ def test_collate_1d(tmp_path, monkeypatch):
 
     with monkeypatch.context() as m:
         monkeypatch.setattr(coadd1d.CoAdd1D, "get_instance", mock_get_instance)
-        monkeypatch.setattr(scripts.collate_1d, "get_archive_subdir", mock_get_subdir)
 
         os.chdir(tmp_path)
         par_file = str(tmp_path / 'collate1d.par')
         
-        # For testing archiving, create a fake pypeit output directory structure
-        # in temp space.
-        temp_spec1d_dir = tmp_path / 'Science'
-        temp_spec1d_dir.mkdir()
-        # Copy spec1d and spec2d
-        shutil.copy2(expanded_spec1d, temp_spec1d_dir)
-        temp_spec1d = os.path.join(temp_spec1d_dir, os.path.basename(expanded_spec1d))
-
-        spec2d_file = expanded_spec1d.replace("spec1d", "spec2d")
-        shutil.copy2(spec2d_file, temp_spec1d_dir)
-        temp_spec2d = os.path.join(temp_spec1d_dir, os.path.basename(spec2d_file))
-
-        # Create fake text files for archiving, and a fake coadd output. We copy the
-        # the spec1d over to the fake output file because the archiving code reads
-        # the header of the coadd output
-
-        temp_spec1d_text = temp_spec1d.replace(".fits", ".txt")
-        temp_pypeit_file = str(tmp_path / "temp.pypeit")
-        with open(temp_spec1d.replace(".fits", ".txt"), "w") as f:
-            print("test data", file=f)
-        shutil.copy2(temp_spec1d_text, temp_pypeit_file)
-
-        temp_coadd_output = str(tmp_path / "SPAT0176-SLIT0175-DET01_KASTb_20150520.fits")
-        shutil.copy2(temp_spec1d, temp_coadd_output)
-
         # Test:
         # * main
         # * creation of collate1d.par
         # * parsing of pixel tolerance
         # * detection of spec2d files and excluding by slit bitmask
-        # * archiving of spec1ds, spec2ds, .txt files, and pypeit files
 
         archive_dir = tmp_path / 'archive'
 
         parsed_args = scripts.collate_1d.Collate1D.parse_args(['--par_outfile', par_file, '--match',
                                                                'pixel', '--tolerance', '3',
-                                                               '--spec1d_files', temp_spec1d,
-                                                               '--exclude_slit_bm', 'BADREDUCE', 
-                                                               '--archive_dir', str(archive_dir)])
+                                                               '--spec1d_files', expanded_spec1d,
+                                                               '--exclude_slit_bm', 'BADREDUCE'])
         assert scripts.collate_1d.Collate1D.main(parsed_args) == 0
         assert os.path.exists(par_file)
 
-        archive_dest_dir = archive_dir / "subdir"
-        assert os.path.exists(archive_dest_dir / os.path.basename(temp_coadd_output))
-        assert os.path.exists(archive_dest_dir / os.path.basename(temp_spec1d))
-        assert os.path.exists(archive_dest_dir / os.path.basename(temp_spec2d))
-        assert os.path.exists(archive_dest_dir / os.path.basename(temp_spec1d_text))
-        assert os.path.exists(archive_dest_dir / os.path.basename(temp_pypeit_file))
-
         # Remove par_file to avoid a warning
         os.unlink(par_file)
         
@@ -527,7 +558,121 @@ def test_collate_1d(tmp_path, monkeypatch):
                                                                '--spec1d_files', expanded_alt_spec1d])
         assert scripts.collate_1d.Collate1D.main(parsed_args) == 0
         
+# pypeit_parse_calib_id is tested in test_runpypeit
 
-# TODO: Include tests for coadd2d, sensfunc, flux_calib
+@cooked_required
+def test_parse_slits():
+    slits_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'shane_kast_blue',
+                              'MasterSlits_A_1_DET01.fits.gz')
+    spec2d_file = os.path.join(os.getenv('PYPEIT_DEV'), 'Cooked', 'Science',
+                              'spec2d_b27-J1217p3905_KASTb_20150520T045733.560.fits')
+
+    # Slits
+    pargs = parse_slits.ParseSlits.parse_args([slits_file])
+    parse_slits.ParseSlits.main(pargs)
+
+    # Spec2d
+    pargs = parse_slits.ParseSlits.parse_args([spec2d_file])
+    parse_slits.ParseSlits.main(pargs)
+    
+def test_flux_calib(tmp_path, monkeypatch):
+
+    # Change to the tmp_path so the fluxing.par file is written there
+    os.chdir(tmp_path)
+
+    # Test the flux_calib script (but not fluxing itself)
+    def mock_get_header(*args, **kwargs):
+        return {"DISPNAME": "600ZD",
+                "PYP_SPEC": "keck_deimos" }
+
+    def mock_get_flux_calib_instance(*args, **kwargs):
+        # The flux_calib caller doesn't use the output, it just
+        # depends on the side effect of fluxing
+        return None 
+
+
+    with monkeypatch.context() as m:
+        monkeypatch.setattr(fits, "getheader", mock_get_header)
+        monkeypatch.setattr(fluxcalibrate.FluxCalibrate, "get_instance", mock_get_flux_calib_instance)
+
+        # Test with a flux file missing "flux end"
+
+        config_file_missing_end = str(tmp_path / "test_flux_calib_missing_end.flux")
+
+        with open(config_file_missing_end, "w") as f:
+            print("flux read", file=f)
+            print(" spec1d_file1.fits sens_file1.fits", file=f)
+            print(" spec1d_file2.fits sens_file2.fits", file=f)
+
+
+        with pytest.raises(PypeItError, match="Missing 'flux end'"):
+            parsed_args = scripts.flux_calib.FluxCalib.parse_args([config_file_missing_end])
+            scripts.flux_calib.FluxCalib.main(parsed_args)
+
+        # Test with a flux file missing the flux block entirely
+        config_file_missing_flux = str(tmp_path / "test_flux_calib_missing_flux.flux")
+        with open(config_file_missing_flux, "w") as f:
+            print(" spec1d_file1.fits sens_file1.fits", file=f)
+            print(" spec1d_file2.fits sens_file2.fits", file=f)
+        
+        with pytest.raises(PypeItError, match="Missing flux block in"):
+            parsed_args = scripts.flux_calib.FluxCalib.parse_args([config_file_missing_flux])
+            scripts.flux_calib.FluxCalib.main(parsed_args)
+
+        # Test 1 sens file with multiple spec1ds
+        config_file_one_to_many = str(tmp_path / "test_flux_calib_1_to_many.flux")
+        with open(config_file_one_to_many, "w") as f:
+            print("flux read", file=f)
+            print(" spec1d_file1.fits sens_file1.fits", file=f)
+            print(" spec1d_file2.fits", file=f)
+            print(" spec1d_file3.fits", file=f)
+            print("flux end", file=f)
+
+        parsed_args = scripts.flux_calib.FluxCalib.parse_args([config_file_one_to_many])
+        assert scripts.flux_calib.FluxCalib.main(parsed_args) == 0
+
+        # Test 1 sens file per spec1d
+        config_file_one_to_one = str(tmp_path / "test_flux_calib_one_to_one.flux")
+        with open(config_file_one_to_one, "w") as f:
+            print("flux read", file=f)
+            print(" spec1d_file1.fits sens_file1.fits", file=f)
+            print(" spec1d_file2.fits sens_file2.fits", file=f)
+            print(" spec1d_file3.fits sens_file1.fits", file=f)
+            print("flux end", file=f)
+
+        parsed_args = scripts.flux_calib.FluxCalib.parse_args([config_file_one_to_one])
+        assert scripts.flux_calib.FluxCalib.main(parsed_args) == 0
+        
+        # Test with no sensfunc, but using an archived sensfunc
+        config_file_use_arxiv = str(tmp_path / "test_flux_calib_use_arxiv.flux")
+        with open(config_file_use_arxiv, "w") as f:
+            print("[fluxcalib]", file=f)
+            print(" use_archived_sens = True", file=f)
+            print("flux read", file=f)
+            print(" spec1d_file1.fits", file=f)
+            print(" spec1d_file2.fits", file=f)
+            print(" spec1d_file3.fits", file=f)
+            print("flux end", file=f)
+
+        parsed_args = scripts.flux_calib.FluxCalib.parse_args([config_file_use_arxiv])
+        assert scripts.flux_calib.FluxCalib.main(parsed_args) == 0
+        
+        
+        # Test with no sensfunc, but it's an error because an archive sensfunc
+        # was not requested
+        config_file_no_sens = str(tmp_path / "test_flux_calib_no_sens.flux")
+        with open(config_file_no_sens, "w") as f:
+            print("flux read", file=f)
+            print(" spec1d_file1.fits", file=f)
+            print(" spec1d_file2.fits", file=f)
+            print(" spec1d_file3.fits", file=f)
+            print("flux end", file=f)
+
+        with pytest.raises(PypeItError, match = 'Invalid format for .flux'):
+            parsed_args = scripts.flux_calib.FluxCalib.parse_args([config_file_no_sens])
+            scripts.flux_calib.FluxCalib.main(parsed_args)
+        
+
+# TODO: Include tests for coadd2d, sensfunc
 
 
